@@ -13,12 +13,13 @@ import re
 import typing as t
 from decimal import Decimal
 from poco.proxy import UIObjectProxy
-from poco.exceptions import PocoNoSuchNodeException
+from poco.exceptions import PocoNoSuchNodeException, PocoTargetTimeout
 
 from capp_ui.platforms import PlatformService
 from capp_ui.mobile_terminals import stop_app
 from capp_ui.libs import SleepWait, LoopFindElement
 from capp_ui.utils import logger, get_ui_object_proxy_attr
+from capp_ui.date_extend import is_later_than_current_time
 from capp_ui.dir import get_images_dir, is_exists, join_path
 from capp_ui.date_extend import get_trip_year_month_day, get_datetime_area, is_public_holiday
 
@@ -71,20 +72,175 @@ class CtripAppService(PlatformService):
         my_button = self.device.poco(type="android.view.ViewGroup", name="ctrip.android.view:id/a", desc="我的")
         my_button.click()
 
-    @SleepWait(wait_time=2)
+    @SleepWait(wait_time=1)
+    def touch_settings(self):
+        """我的主页，点击【设置】"""
+        setting_button = self.device.poco(type="android.widget.ImageView", name="ctrip.android.view:id/a", desc="设置")
+        setting_button.click()
+
+    @SleepWait(wait_time=1)
+    def touch_logout_user(self):
+        """在设置界面，点击【退出登录】"""
+        self.device.quick_slide_screen(duration=1)
+        try:
+            logout_user = self.device.poco(
+                type="android.widget.Button", name="ctrip.android.view:id/a", text="退出登录"
+            )
+            if logout_user.exists() is True:
+                logout_user.click()
+        except (PocoNoSuchNodeException,):
+            pass
+
+    @SleepWait(wait_time=1)
+    def touch_submit_logout(self):
+        """退出登录弹框，点击【确定】"""
+        try:
+            logout_user = self.device.poco(
+                type="android.widget.TextView", name="ctrip.android.view:id/a", text="确定"
+            )
+            if logout_user.exists() is True:
+                logout_user.click()
+        except (PocoNoSuchNodeException,):
+            pass
+
+    @SleepWait(wait_time=1)
     def touch_unpaid(self) -> None:
         """进入my主页后，点击【待付款】"""
-        unpaid_button = self.device.poco(type="android.widget.TextView", name="ctrip.android.view:id/a", text="待付款")
-        unpaid_button.click()
+        try:
+            # 登录后的待付款元素定位
+            login_unpaid_button = self.device.poco(
+                type="android.widget.TextView", name="ctrip.android.view:id/a", text="待付款"
+            )
+            # 未登录的待付款元素定位
+            logout_unpaid_button = self.device.poco(
+                type="android.widget.TextView", name="android.widget.TextView", text="待付款"
+            )
+            if login_unpaid_button.exists() is True:
+                login_unpaid_button.click()
+            elif logout_unpaid_button.exists() is True:
+                logout_unpaid_button.click()
+            else:
+                logger.warning("没有找到待付款按钮")
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
 
-    @SleepWait(wait_time=2)
-    def select_search_box(self, ctrip_order_id: str) -> None:
+    @SleepWait(wait_time=1)
+    def touch_list_page_search_box(self) -> None:
         """进入待付款列表页，点击【搜索框】"""
-        search_box = self.device.poco(type="android.widget.TextView", name="ctrip.android.view:id/a", text="搜索订单")
-        search_box.click()
-        search_box.set_text(ctrip_order_id)
-        # 模拟键盘按下回车键（keyCode为66表示回车键）
-        self.device.poco.device.input(keyevent="66")
+        try:
+            search_box = self.device.poco(
+                type="android.widget.TextView", name="android.widget.TextView", text="搜索订单"
+            )
+            search_box.click()
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+
+    @SleepWait(wait_time=1)
+    def touch_search_page_search_box(self, ctrip_order_id: str) -> None:
+        """进入搜索页，点击【搜索框】"""
+        try:
+            search_box = self.device.poco(
+                type="android.widget.EditText", name="android.widget.EditText", text="输入城市名/订单号 搜索订单"
+            )
+            search_box.click()
+            search_box.set_text(ctrip_order_id)
+            # 模拟键盘按下回车键（keyCode为66表示回车键）
+            # self.device.poco.device.input(keyevent="66")
+            self.device.keyevent(keyname="66")
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+
+    @SleepWait(wait_time=1)
+    def is_exist_to_payment_at_list_page(self) -> bool:
+        """检查搜索列表界面，是否存在【去支付】按钮"""
+        flag = False
+        try:
+            search_box = self.device.poco(type="android.widget.TextView", name="android.widget.TextView", text="去支付")
+            if search_box.exists() is True:
+                flag = True
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+        return flag
+
+    @SleepWait(wait_time=1)
+    def get_order_status(self) -> str:
+        """检查搜索列表界面，获取订单的状态"""
+        status = None
+        try:
+            unpaid_status = self.device.poco(
+                type="android.widget.TextView", name="android.widget.TextView", text="去支付"
+            )
+            cancel_status = self.device.poco(
+                type="android.widget.TextView", name="android.widget.TextView", text="已取消"
+            )
+            if unpaid_status.exists() is True:
+                status = "待支付"
+            elif cancel_status.exists() is True:
+                status = "已取消"
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+        return status
+
+    def is_order_detail_page(self) -> bool:
+        """是否在订单详情界面"""
+        flag = False
+        try:
+            order_detail_page_1 = self.device.poco(
+                type="android.widget.TextView", name="header_Text_订单详情", text="订单详情"
+            )
+            order_detail_page_2 = self.device.poco(
+                type="android.widget.TextView", name="operateBtnList_Text_我要退订", text="我要退订"
+            )
+            order_detail_page_3 = self.device.poco(
+                type="android.view.ViewGroup", name="CusHeaderView_TouchableOpacity_leftIconWrap",
+                desc="CusHeaderView_TouchableOpacity_leftIconWrap"
+            )
+            if (order_detail_page_1.exists() is True or order_detail_page_2.exists() is True or
+                    order_detail_page_3.exists() is True):
+                flag = True
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+        return flag
+
+    @SleepWait(wait_time=1)
+    def touch_go_back_to_unpaid_list_page_by_search_page(self) -> None:
+        """从搜索界面退回到【待支付】列表页"""
+        try:
+            go_back = self.device.poco(type="android.widget.TextView", name="icon_back")
+            if go_back.exists() is True:
+                go_back.click()
+                go_back.click()
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+
+    @SleepWait(wait_time=1)
+    def touch_go_back_to_search_page_page_by_cancel_page(self) -> None:
+        """从订单撤销页返回到订单待支付列表页"""
+        try:
+            go_back = self.device.poco(
+                type="android.view.ViewGroup", name="CusHeaderView_TouchableOpacity_leftIconWrap",
+                desc="CusHeaderView_TouchableOpacity_leftIconWrap"
+            )
+            if go_back.exists() is True:
+                go_back.click()
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
 
     @SleepWait(wait_time=1)
     def touch_to_payment_at_list_page(self) -> None:
@@ -92,6 +248,83 @@ class CtripAppService(PlatformService):
         search_box = self.device.poco(type="android.widget.TextView", name="android.widget.TextView", text="去支付")
         search_box.click()
 
+    @SleepWait(wait_time=1)
+    def is_cancel_order(self, out_total_price: str) -> tuple:
+        """在订单详情页，判断是否需要取消订单"""
+        flag = False
+        remark = None
+        try:
+            is_cancel = self.device.poco(
+                type="android.widget.TextView", name="android.widget.TextView", textMatches=r"^请在\d+:\d+前支付.*"
+            )
+            if is_cancel.exists() is True:
+                text = is_cancel.get_text()
+                time_match = re.search(r'(\d{2}:\d{2})', text)
+                amount_match = re.search(r'¥(\d+)', text)
+                if time_match and amount_match:
+                    time_str = time_match.group(1)
+                    string = "从订单获取到的过期时间为：{}".format(time_str)
+                    is_later = is_later_than_current_time(time_str=time_str, minutes=5)
+                    if is_later is False:
+                        flag = True
+                        remark = "支付时间少于5分钟"
+                        string = string + "，" + remark
+                    else:
+                        amount_str = amount_match.group(1)
+                        string = "从订单获取到的支付金额为：{}，实际出票总订单价：{}".format(amount_str, out_total_price)
+                        if Decimal(amount_str) > Decimal(out_total_price):
+                            flag = True
+                            remark = "航班已变价"
+                            string = string + "，" + remark
+                else:
+                    string = "从元素的文案<{}>提取时间与金额信息有异常".format(text)
+            else:
+                string = "元素定位存在异常，订单详情页没有找到订单支付金额和过期时间"
+            print(string)
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+        return flag, remark
+
+    @SleepWait(wait_time=1)
+    def touch_cancel_order(self) -> None:
+        try:
+            cancel_order = self.device.poco(
+                type="android.widget.TextView", name="operateBtnList_Text_取消订单", text="取消订单"
+            )
+            cancel_order.click()
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+
+    @SleepWait(wait_time=1)
+    def touch_submit_cancel_order(self) -> None:
+        """再次确认是否要取消订单"""
+        try:
+            submit_cancel_order = self.device.poco(
+                type="android.widget.TextView", name="Button_Text_取消订单", text="取消订单"
+            )
+            submit_cancel_order.click()
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+
+    @SleepWait(wait_time=1)
+    def touch_know_the_cancel_order(self) -> None:
+        """确认取消后，会有一个【知道了】的小弹框"""
+        try:
+            submit_cancel_order = self.device.poco(
+                type="android.widget.TextView", name="Button_Text_知道了", text="知道了"
+            )
+            submit_cancel_order.click()
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
+            pass
+        except Exception as e:
+            logger.error(e)
+
     @SleepWait(wait_time=2)
     def touch_to_payment_at_order_detail(self) -> None:
         """在订单详情页，点击【去支付】"""
@@ -497,14 +730,23 @@ class CtripAppService(PlatformService):
     def is_need_login(self) -> bool:
         flag = False
         try:
-            login = self.device.get_po(
+            login_page = self.device.get_po(
                 type="android.widget.TextView", name="ctrip.android.view:id/a", text="手机验证码登录"
             )
-            if login.exists() is True:
+            my_login = self.device.get_po(
+                type="android.widget.Button", name="ctrip.android.view:id/a", text="登录/注册"
+            )
+            if login_page.exists() is True:
                 logger.warning("手机app已经跳转至登录界面，需要做登录操作.")
                 flag = True
-        except (PocoNoSuchNodeException, Exception):
+            elif my_login.exists() is True:
+                my_login.click()
+                logger.warning("手机app已经跳转至登录界面，需要做登录操作.")
+                flag = True
+        except (PocoNoSuchNodeException, PocoTargetTimeout):
             pass
+        except Exception as e:
+            logger.error(e)
         return flag
 
     def select_agree_service_agreement(self) -> None:
@@ -537,8 +779,9 @@ class CtripAppService(PlatformService):
         )
         # password_poco.click()
         password_poco.set_text(password)
-        file_name = join_path([get_images_dir(), "键盘隐藏.png"])
-        self.device.hide_keyword(file_name=file_name)
+        # file_name = join_path([get_images_dir(), "键盘隐藏.png"])
+        # self.device.hide_keyword(file_name=file_name)
+        self.device.quick_slide_screen(duration=0.5)
 
     def touch_login(self) -> None:
         """点击【登录】"""
@@ -1027,7 +1270,7 @@ class CtripAppService(PlatformService):
                                                                        str) else -9999999999.9999999999
         return deduction_amount
 
-    @SleepWait(wait_time=10)
+    @SleepWait(wait_time=5)
     def enter_payment_pass(self, payment_pass: str) -> None:
         """
         请输入支付密码
@@ -1042,7 +1285,7 @@ class CtripAppService(PlatformService):
             else:
                 raise ValueError("文件<{}>缺失...", format(file_name))
 
-    @SleepWait(wait_time=10)
+    @SleepWait(wait_time=1)
     def is_balance(self, payment_card: str) -> bool:
         """
         判断是否出现余额不足的小弹框
@@ -1062,7 +1305,7 @@ class CtripAppService(PlatformService):
             logger.warning("没有找到余额不足小弹框，接下来更换支付方式，继续支付.")
         return flag
 
-    @SleepWait(wait_time=10)
+    @SleepWait(wait_time=1)
     def is_payment_complete(self) -> bool:
         """
         判断是否限额，余额不足或其他异常，没有异常的话，输入密码后，会出现【完成】小弹框界面
@@ -1104,7 +1347,7 @@ class CtripAppService(PlatformService):
             logger.error(str(e))
         return payment_amount
 
-    @SleepWait(wait_time=20)
+    @SleepWait(wait_time=5)
     def touch_payment_complete(self) -> None:
         """在支付成功界面，点击【完成】"""
         try:
@@ -1233,6 +1476,8 @@ class CtripAppService(PlatformService):
             logger.warning("出票中，没有查找到行程单.")
         return itinerary_id
 
+    """
     # 为了调式方便，暂时注释
     def __del__(self) -> None:
         self.stop()
+    """
